@@ -2,6 +2,10 @@ package slicesdelete
 
 var g struct{ f []int }
 
+func h() []int { return []int{} }
+
+var ch chan []int
+
 func slicesdelete(test, other []byte, i int) {
 	const k = 1
 	_ = append(test[:i], test[i+1:]...) // want "Replace append with slices.Delete"
@@ -26,6 +30,10 @@ func slicesdelete(test, other []byte, i int) {
 
 	_ = append(g.f[:i], g.f[i+k:]...) // want "Replace append with slices.Delete"
 
+	_ = append(h()[:i], h()[i+1:]...) // potentially has side effects
+
+	_ = append((<-ch)[:i], (<-ch)[i+1:]...) // has side effects
+
 	_ = append(test[:3], test[i+1:]...) // cannot verify a < b
 
 	_ = append(test[:i-4], test[i-1:]...) // want "Replace append with slices.Delete"
