@@ -24,19 +24,18 @@ import (
 	"sync"
 	"time"
 
+	"golang.org/x/tools/gopls/internal/cache"
 	"golang.org/x/tools/gopls/internal/debug/log"
-	"golang.org/x/tools/gopls/internal/lsp/cache"
-	"golang.org/x/tools/gopls/internal/lsp/protocol"
+	label1 "golang.org/x/tools/gopls/internal/label"
+	"golang.org/x/tools/gopls/internal/protocol"
 	"golang.org/x/tools/gopls/internal/util/bug"
 	"golang.org/x/tools/internal/event"
 	"golang.org/x/tools/internal/event/core"
 	"golang.org/x/tools/internal/event/export"
 	"golang.org/x/tools/internal/event/export/metric"
-	"golang.org/x/tools/internal/event/export/ocagent"
 	"golang.org/x/tools/internal/event/export/prometheus"
 	"golang.org/x/tools/internal/event/keys"
 	"golang.org/x/tools/internal/event/label"
-	"golang.org/x/tools/internal/event/tag"
 )
 
 type contextKeyType int
@@ -51,14 +50,11 @@ type Instance struct {
 	Logfile       string
 	StartTime     time.Time
 	ServerAddress string
-	Workdir       string
-	OCAgentConfig string
 
 	LogWriter io.Writer
 
 	exporter event.Exporter
 
-	ocagent    *ocagent.Exporter
 	prometheus *prometheus.Exporter
 	rpcs       *Rpcs
 	traces     *traces
@@ -281,23 +277,23 @@ func cmdline(w http.ResponseWriter, r *http.Request) {
 	pprof.Cmdline(fake, r)
 }
 
-func (i *Instance) getCache(r *http.Request) interface{} {
+func (i *Instance) getCache(r *http.Request) any {
 	return i.State.Cache(path.Base(r.URL.Path))
 }
 
-func (i *Instance) getAnalysis(r *http.Request) interface{} {
+func (i *Instance) getAnalysis(r *http.Request) any {
 	return i.State.Analysis()
 }
 
-func (i *Instance) getSession(r *http.Request) interface{} {
+func (i *Instance) getSession(r *http.Request) any {
 	return i.State.Session(path.Base(r.URL.Path))
 }
 
-func (i *Instance) getClient(r *http.Request) interface{} {
+func (i *Instance) getClient(r *http.Request) any {
 	return i.State.Client(path.Base(r.URL.Path))
 }
 
-func (i *Instance) getServer(r *http.Request) interface{} {
+func (i *Instance) getServer(r *http.Request) any {
 	i.State.mu.Lock()
 	defer i.State.mu.Unlock()
 	id := path.Base(r.URL.Path)
@@ -309,11 +305,7 @@ func (i *Instance) getServer(r *http.Request) interface{} {
 	return nil
 }
 
-func (i *Instance) getView(r *http.Request) interface{} {
-	return i.State.View(path.Base(r.URL.Path))
-}
-
-func (i *Instance) getFile(r *http.Request) interface{} {
+func (i *Instance) getFile(r *http.Request) any {
 	identifier := path.Base(r.URL.Path)
 	sid := path.Base(path.Dir(r.URL.Path))
 	s := i.State.Session(sid)
@@ -329,7 +321,7 @@ func (i *Instance) getFile(r *http.Request) interface{} {
 	return nil
 }
 
-func (i *Instance) getInfo(r *http.Request) interface{} {
+func (i *Instance) getInfo(r *http.Request) any {
 	buf := &bytes.Buffer{}
 	i.PrintServerInfo(r.Context(), buf)
 	return template.HTML(buf.String())
@@ -345,7 +337,7 @@ func (i *Instance) AddService(s protocol.Server, session *cache.Session) {
 	stdlog.Printf("unable to find a Client to add the protocol.Server to")
 }
 
-func getMemory(_ *http.Request) interface{} {
+func getMemory(_ *http.Request) any {
 	var m runtime.MemStats
 	runtime.ReadMemStats(&m)
 	return m
@@ -368,17 +360,11 @@ func GetInstance(ctx context.Context) *Instance {
 
 // WithInstance creates debug instance ready for use using the supplied
 // configuration and stores it in the returned context.
-func WithInstance(ctx context.Context, workdir, agent string) context.Context {
+func WithInstance(ctx context.Context) context.Context {
 	i := &Instance{
-		StartTime:     time.Now(),
-		Workdir:       workdir,
-		OCAgentConfig: agent,
+		StartTime: time.Now(),
 	}
 	i.LogWriter = os.Stderr
-	ocConfig := ocagent.Discover()
-	//TODO: we should not need to adjust the discovered configuration
-	ocConfig.Address = i.OCAgentConfig
-	i.ocagent = ocagent.Connect(ocConfig)
 	i.prometheus = prometheus.New()
 	i.rpcs = &Rpcs{}
 	i.traces = &traces{}
@@ -442,10 +428,10 @@ func (i *Instance) Serve(ctx context.Context, addr string) (string, error) {
 	if strings.HasSuffix(i.debugAddress, ":0") {
 		stdlog.Printf("debug server listening at http://localhost:%d", port)
 	}
-	event.Log(ctx, "Debug serving", tag.Port.Of(port))
+	event.Log(ctx, "Debug serving", label1.Port.Of(port))
 	go func() {
 		mux := http.NewServeMux()
-		mux.HandleFunc("/", render(MainTmpl, func(*http.Request) interface{} { return i }))
+		mux.HandleFunc("/", render(MainTmpl, func(*http.Request) any { return i }))
 		mux.HandleFunc("/debug/", render(DebugTmpl, nil))
 		mux.HandleFunc("/debug/pprof/", pprof.Index)
 		mux.HandleFunc("/debug/pprof/cmdline", cmdline)
@@ -464,7 +450,6 @@ func (i *Instance) Serve(ctx context.Context, addr string) (string, error) {
 		mux.HandleFunc("/analysis/", render(AnalysisTmpl, i.getAnalysis))
 		mux.HandleFunc("/cache/", render(CacheTmpl, i.getCache))
 		mux.HandleFunc("/session/", render(SessionTmpl, i.getSession))
-		mux.HandleFunc("/view/", render(ViewTmpl, i.getView))
 		mux.HandleFunc("/client/", render(ClientTmpl, i.getClient))
 		mux.HandleFunc("/server/", render(ServerTmpl, i.getServer))
 		mux.HandleFunc("/file/", render(FileTmpl, i.getFile))
@@ -548,9 +533,6 @@ func messageType(l log.Level) protocol.MessageType {
 
 func makeInstanceExporter(i *Instance) event.Exporter {
 	exporter := func(ctx context.Context, ev core.Event, lm label.Map) context.Context {
-		if i.ocagent != nil {
-			ctx = i.ocagent.ProcessEvent(ctx, ev, lm)
-		}
 		if i.prometheus != nil {
 			ctx = i.prometheus.ProcessEvent(ctx, ev, lm)
 		}
@@ -564,27 +546,27 @@ func makeInstanceExporter(i *Instance) event.Exporter {
 			if s := cache.KeyCreateSession.Get(ev); s != nil {
 				i.State.addClient(s)
 			}
-			if sid := tag.NewServer.Get(ev); sid != "" {
+			if sid := label1.NewServer.Get(ev); sid != "" {
 				i.State.updateServer(&Server{
 					ID:           sid,
-					Logfile:      tag.Logfile.Get(ev),
-					DebugAddress: tag.DebugAddress.Get(ev),
-					GoplsPath:    tag.GoplsPath.Get(ev),
-					ClientID:     tag.ClientID.Get(ev),
+					Logfile:      label1.Logfile.Get(ev),
+					DebugAddress: label1.DebugAddress.Get(ev),
+					GoplsPath:    label1.GoplsPath.Get(ev),
+					ClientID:     label1.ClientID.Get(ev),
 				})
 			}
 			if s := cache.KeyShutdownSession.Get(ev); s != nil {
 				i.State.dropClient(s)
 			}
-			if sid := tag.EndServer.Get(ev); sid != "" {
+			if sid := label1.EndServer.Get(ev); sid != "" {
 				i.State.dropServer(sid)
 			}
 			if s := cache.KeyUpdateSession.Get(ev); s != nil {
 				if c := i.State.Client(s.ID()); c != nil {
-					c.DebugAddress = tag.DebugAddress.Get(ev)
-					c.Logfile = tag.Logfile.Get(ev)
-					c.ServerID = tag.ServerID.Get(ev)
-					c.GoplsPath = tag.GoplsPath.Get(ev)
+					c.DebugAddress = label1.DebugAddress.Get(ev)
+					c.Logfile = label1.Logfile.Get(ev)
+					c.ServerID = label1.ServerID.Get(ev)
+					c.GoplsPath = label1.GoplsPath.Get(ev)
 				}
 			}
 		}
@@ -601,11 +583,11 @@ func makeInstanceExporter(i *Instance) event.Exporter {
 	return exporter
 }
 
-type dataFunc func(*http.Request) interface{}
+type dataFunc func(*http.Request) any
 
 func render(tmpl *template.Template, fun dataFunc) func(http.ResponseWriter, *http.Request) {
 	return func(w http.ResponseWriter, r *http.Request) {
-		var data interface{}
+		var data any
 		if fun != nil {
 			data = fun(r)
 		}
@@ -646,12 +628,17 @@ var BaseTemplate = template.Must(template.New("").Parse(`
 	width:6rem;
 }
 td.value {
-  text-align: right;
+	text-align: right;
 }
 ul.spans {
 	font-family: monospace;
 	font-size:   85%;
 }
+body {
+	font-family: sans-serif;
+	font-size: 1rem;
+	line-height: normal;
+}
 </style>
 {{block "head" .}}{{end}}
 </head>
@@ -676,7 +663,6 @@ Unknown page
 {{define "clientlink"}}<a href="/client/{{.}}">Client {{.}}</a>{{end}}
 {{define "serverlink"}}<a href="/server/{{.}}">Server {{.}}</a>{{end}}
 {{define "sessionlink"}}<a href="/session/{{.}}">Session {{.}}</a>{{end}}
-{{define "viewlink"}}<a href="/view/{{.}}">View {{.}}</a>{{end}}
 `)).Funcs(template.FuncMap{
 	"fuint64":  fuint64,
 	"fuint32":  fuint32,
@@ -708,7 +694,7 @@ Unknown page
 })
 
 var MainTmpl = template.Must(template.Must(BaseTemplate.Clone()).Parse(`
-{{define "title"}}GoPls server information{{end}}
+{{define "title"}}Gopls server information{{end}}
 {{define "body"}}
 <h2>Caches</h2>
 <ul>{{range .State.Caches}}<li>{{template "cachelink" .ID}}</li>{{end}}</ul>
@@ -724,7 +710,7 @@ var MainTmpl = template.Must(template.Must(BaseTemplate.Clone()).Parse(`
 `))
 
 var InfoTmpl = template.Must(template.Must(BaseTemplate.Clone()).Parse(`
-{{define "title"}}GoPls version information{{end}}
+{{define "title"}}Gopls version information{{end}}
 {{define "body"}}
 {{.}}
 {{end}}
@@ -773,6 +759,13 @@ var CacheTmpl = template.Must(template.Must(BaseTemplate.Clone()).Parse(`
 {{define "body"}}
 <h2>memoize.Store entries</h2>
 <ul>{{range $k,$v := .MemStats}}<li>{{$k}} - {{$v}}</li>{{end}}</ul>
+<h2>File stats</h2>
+<p>
+{{- $stats := .FileStats -}}
+Total: <b>{{$stats.Total}}</b><br>
+Largest: <b>{{$stats.Largest}}</b><br>
+Errors: <b>{{$stats.Errs}}</b><br>
+</p>
 {{end}}
 `))
 
@@ -791,29 +784,6 @@ Using session: <b>{{template "sessionlink" .Session.ID}}</b><br>
 {{if .DebugAddress}}Debug this client at: <a href="http://{{localAddress .DebugAddress}}">{{localAddress .DebugAddress}}</a><br>{{end}}
 Logfile: {{.Logfile}}<br>
 Gopls Path: {{.GoplsPath}}<br>
-<h2>Diagnostics</h2>
-{{/*Service: []protocol.Server; each server has map[uri]fileReports;
-	each fileReport: map[diagnosticSoure]diagnosticReport
-	diagnosticSource is one of 5 source
-	diagnosticReport: snapshotID and map[hash]*source.Diagnostic
-	sourceDiagnostic: struct {
-		Range    protocol.Range
-		Message  string
-		Source   string
-		Code     string
-		CodeHref string
-		Severity protocol.DiagnosticSeverity
-		Tags     []protocol.DiagnosticTag
-
-		Related []RelatedInformation
-	}
-	RelatedInformation: struct {
-		URI     protocol.DocumentURI
-		Range   protocol.Range
-		Message string
-	}
-	*/}}
-<ul>{{range $k, $v := .Service.Diagnostics}}<li>{{$k}}:<ol>{{range $v}}<li>{{.}}</li>{{end}}</ol></li>{{end}}</ul>
 {{end}}
 `))
 
@@ -831,7 +801,16 @@ var SessionTmpl = template.Must(template.Must(BaseTemplate.Clone()).Parse(`
 {{define "body"}}
 From: <b>{{template "cachelink" .Cache.ID}}</b><br>
 <h2>Views</h2>
-<ul>{{range .Views}}<li>{{.Name}} is {{template "viewlink" .ID}} in {{.Folder}}</li>{{end}}</ul>
+<ul>{{range .Views}}
+{{- $envOverlay := .EnvOverlay -}}
+<li>ID: <b>{{.ID}}</b><br>
+Type: <b>{{.Type}}</b><br>
+Root: <b>{{.Root}}</b><br>
+{{- if $envOverlay}}
+Env overlay: <b>{{$envOverlay}})</b><br>
+{{end -}}
+Folder: <b>{{.Folder.Name}}:{{.Folder.Dir}}</b></li>
+{{end}}</ul>
 <h2>Overlays</h2>
 {{$session := .}}
 <ul>{{range .Overlays}}
@@ -841,14 +820,6 @@ From: <b>{{template "cachelink" .Cache.ID}}</b><br>
 {{end}}
 `))
 
-var ViewTmpl = template.Must(template.Must(BaseTemplate.Clone()).Parse(`
-{{define "title"}}View {{.ID}}{{end}}
-{{define "body"}}
-Name: <b>{{.Name}}</b><br>
-Folder: <b>{{.Folder}}</b><br>
-{{end}}
-`))
-
 var FileTmpl = template.Must(template.Must(BaseTemplate.Clone()).Parse(`
 {{define "title"}}Overlay {{.Identity.Hash}}{{end}}
 {{define "body"}}
