@@ -38,6 +38,7 @@ import (
 	gastutil "golang.org/x/tools/gopls/internal/util/astutil"
 	"golang.org/x/tools/gopls/internal/util/bug"
 	"golang.org/x/tools/gopls/internal/util/safetoken"
+	internalastutil "golang.org/x/tools/internal/astutil"
 	"golang.org/x/tools/internal/event"
 	"golang.org/x/tools/internal/stdlib"
 	"golang.org/x/tools/internal/tokeninternal"
@@ -286,6 +287,10 @@ func hover(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle, pp pro
 		}
 	}
 
+	// By convention, we qualify hover information relative to the package
+	// from which the request originated.
+	qual := typesinternal.FileQualifier(pgf.File, pkg.Types())
+
 	// Handle hover over identifier.
 
 	// The general case: compute hover information for the object referenced by
@@ -304,10 +309,6 @@ func hover(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle, pp pro
 		hoverRange = &rng
 	}
 
-	// By convention, we qualify hover information relative to the package
-	// from which the request originated.
-	qual := typesinternal.FileQualifier(pgf.File, pkg.Types())
-
 	// Handle type switch identifiers as a special case, since they don't have an
 	// object.
 	//
@@ -343,6 +344,42 @@ func hover(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle, pp pro
 
 	// By default, types.ObjectString provides a reasonable signature.
 	signature := objectString(obj, qual, declPos, declPGF.Tok, spec)
+
+	// When hovering over a reference to a promoted struct field,
+	// show the implicitly selected intervening fields.
+	cur, ok := pgf.Cursor.FindByPos(pos, pos)
+	if !ok {
+		return protocol.Range{}, nil, fmt.Errorf("Invalid hover position, failed to get cursor")
+	}
+	if obj, ok := obj.(*types.Var); ok && obj.IsField() {
+		if selExpr, ok := cur.Parent().Node().(*ast.SelectorExpr); ok {
+			sel := pkg.TypesInfo().Selections[selExpr]
+			if len(sel.Index()) > 1 {
+				var buf bytes.Buffer
+				buf.WriteString(" // through ")
+				t := typesinternal.Unpointer(sel.Recv())
+				for i, index := range sel.Index()[:len(sel.Index())-1] {
+					if i > 0 {
+						buf.WriteString(", ")
+					}
+					field := typesinternal.Unpointer(t.Underlying()).(*types.Struct).Field(index)
+					t = field.Type()
+					// Inv: fieldType is N or *N for some NamedOrAlias type N.
+					if ptr, ok := t.(*types.Pointer); ok {
+						buf.WriteString("*")
+						t = ptr.Elem()
+					}
+					// Be defensive in case of ill-typed code:
+					if named, ok := t.(typesinternal.NamedOrAlias); ok {
+						buf.WriteString(named.Obj().Name())
+					}
+				}
+				// Update signature to include embedded struct info.
+				signature += buf.String()
+			}
+		}
+	}
+
 	singleLineSignature := signature
 
 	// Display struct tag for struct fields at the end of the signature.
@@ -614,7 +651,7 @@ func hover(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle, pp pro
 		linkPath = ""
 	} else if linkMeta.Module != nil && linkMeta.Module.Version != "" {
 		mod := linkMeta.Module
-		linkPath = strings.Replace(linkPath, mod.Path, mod.Path+"@"+mod.Version, 1)
+		linkPath = strings.Replace(linkPath, mod.Path, cache.ResolvedString(mod), 1)
 	}
 
 	var footer string
@@ -1502,16 +1539,10 @@ func findDeclInfo(files []*ast.File, pos token.Pos) (decl ast.Decl, spec ast.Spe
 	stack := make([]ast.Node, 0, 20)
 
 	// Allocate the closure once, outside the loop.
-	f := func(n ast.Node) bool {
+	f := func(n ast.Node, stack []ast.Node) bool {
 		if found {
 			return false
 		}
-		if n != nil {
-			stack = append(stack, n) // push
-		} else {
-			stack = stack[:len(stack)-1] // pop
-			return false
-		}
 
 		// Skip subtrees (incl. files) that don't contain the search point.
 		if !(n.Pos() <= pos && pos < n.End()) {
@@ -1596,7 +1627,7 @@ func findDeclInfo(files []*ast.File, pos token.Pos) (decl ast.Decl, spec ast.Spe
 		return true
 	}
 	for _, file := range files {
-		ast.Inspect(file, f)
+		internalastutil.PreorderStack(file, stack, f)
 		if found {
 			return decl, spec, field
 		}
