@@ -10,6 +10,7 @@ import (
 	"encoding/json"
 	"errors"
 	"fmt"
+	"maps"
 	"math/rand/v2"
 	"os"
 	"path"
@@ -108,6 +109,14 @@ type EditorConfig struct {
 	// To explicitly send no workspace folders, use an empty (non-nil) slice.
 	WorkspaceFolders []string
 
+	// NoDefaultWorkspaceFiles is used to specify whether the fake editor
+	// should give a default workspace folder when WorkspaceFolders is nil.
+	// When it's true, the editor will pass original WorkspaceFolders as is to the LSP server.
+	NoDefaultWorkspaceFiles bool
+
+	// RelRootPath is the root path which will be converted to rootUri to configure on the LSP server.
+	RelRootPath string
+
 	// Whether to edit files with windows line endings.
 	WindowsLineEndings bool
 
@@ -253,12 +262,8 @@ func (e *Editor) Client() *Client {
 // makeSettings builds the settings map for use in LSP settings RPCs.
 func makeSettings(sandbox *Sandbox, config EditorConfig, scopeURI *protocol.URI) map[string]any {
 	env := make(map[string]string)
-	for k, v := range sandbox.GoEnv() {
-		env[k] = v
-	}
-	for k, v := range config.Env {
-		env[k] = v
-	}
+	maps.Copy(env, sandbox.GoEnv())
+	maps.Copy(env, config.Env)
 	for k, v := range env {
 		v = strings.ReplaceAll(v, "$SANDBOX_WORKDIR", sandbox.Workdir.RootURI().Path())
 		env[k] = v
@@ -299,9 +304,7 @@ func makeSettings(sandbox *Sandbox, config EditorConfig, scopeURI *protocol.URI)
 			}
 		}
 		if closestSettings != nil {
-			for k, v := range closestSettings {
-				settings[k] = v
-			}
+			maps.Copy(settings, closestSettings)
 		}
 	}
 
@@ -322,8 +325,9 @@ func (e *Editor) initialize(ctx context.Context) error {
 		Version: "v1.0.0",
 	}
 	params.InitializationOptions = makeSettings(e.sandbox, config, nil)
-	params.WorkspaceFolders = makeWorkspaceFolders(e.sandbox, config.WorkspaceFolders)
 
+	params.WorkspaceFolders = makeWorkspaceFolders(e.sandbox, config.WorkspaceFolders, config.NoDefaultWorkspaceFiles)
+	params.RootURI = protocol.DocumentURI(makeRootURI(e.sandbox, config.RelRootPath))
 	capabilities, err := clientCapabilities(config)
 	if err != nil {
 		return fmt.Errorf("unmarshalling EditorConfig.CapabilitiesJSON: %v", err)
@@ -434,12 +438,7 @@ func marshalUnmarshal[T any](v any) (T, error) {
 
 // HasCommand reports whether the connected server supports the command with the given ID.
 func (e *Editor) HasCommand(cmd command.Command) bool {
-	for _, command := range e.serverCapabilities.ExecuteCommandProvider.Commands {
-		if command == cmd.String() {
-			return true
-		}
-	}
-	return false
+	return slices.Contains(e.serverCapabilities.ExecuteCommandProvider.Commands, cmd.String())
 }
 
 // Examples: https://www.iana.org/assignments/uri-schemes/uri-schemes.xhtml
@@ -447,7 +446,10 @@ var uriRE = regexp.MustCompile(`^[a-z][a-z0-9+\-.]*://\S+`)
 
 // makeWorkspaceFolders creates a slice of workspace folders to use for
 // this editing session, based on the editor configuration.
-func makeWorkspaceFolders(sandbox *Sandbox, paths []string) (folders []protocol.WorkspaceFolder) {
+func makeWorkspaceFolders(sandbox *Sandbox, paths []string, useEmpty bool) (folders []protocol.WorkspaceFolder) {
+	if len(paths) == 0 && useEmpty {
+		return nil
+	}
 	if len(paths) == 0 {
 		paths = []string{string(sandbox.Workdir.RelativeTo)}
 	}
@@ -466,6 +468,14 @@ func makeWorkspaceFolders(sandbox *Sandbox, paths []string) (folders []protocol.
 	return folders
 }
 
+func makeRootURI(sandbox *Sandbox, path string) string {
+	uri := path
+	if !uriRE.MatchString(path) { // relative file path
+		uri = string(sandbox.Workdir.URI(path))
+	}
+	return uri
+}
+
 // onFileChanges is registered to be called by the Workdir on any writes that
 // go through the Workdir API. It is called synchronously by the Workdir.
 func (e *Editor) onFileChanges(ctx context.Context, evts []protocol.FileEvent) {
@@ -1159,11 +1169,8 @@ func (e *Editor) ExecuteCommand(ctx context.Context, params *protocol.ExecuteCom
 	var match bool
 	if e.serverCapabilities.ExecuteCommandProvider != nil {
 		// Ensure that this command was actually listed as a supported command.
-		for _, command := range e.serverCapabilities.ExecuteCommandProvider.Commands {
-			if command == params.Command {
-				match = true
-				break
-			}
+		if slices.Contains(e.serverCapabilities.ExecuteCommandProvider.Commands, params.Command) {
+			match = true
 		}
 	}
 	if !match {
@@ -1309,6 +1316,19 @@ func (e *Editor) Completion(ctx context.Context, loc protocol.Location) (*protoc
 	return completions, nil
 }
 
+func (e *Editor) DidCreateFiles(ctx context.Context, files ...protocol.DocumentURI) error {
+	if e.Server == nil {
+		return nil
+	}
+	params := &protocol.CreateFilesParams{}
+	for _, file := range files {
+		params.Files = append(params.Files, protocol.FileCreate{
+			URI: string(file),
+		})
+	}
+	return e.Server.DidCreateFiles(ctx, params)
+}
+
 func (e *Editor) SetSuggestionInsertReplaceMode(_ context.Context, useReplaceMode bool) {
 	e.mu.Lock()
 	defer e.mu.Unlock()
@@ -1632,8 +1652,8 @@ func (e *Editor) ChangeWorkspaceFolders(ctx context.Context, folders []string) e
 	config := e.Config()
 
 	// capture existing folders so that we can compute the change.
-	oldFolders := makeWorkspaceFolders(e.sandbox, config.WorkspaceFolders)
-	newFolders := makeWorkspaceFolders(e.sandbox, folders)
+	oldFolders := makeWorkspaceFolders(e.sandbox, config.WorkspaceFolders, config.NoDefaultWorkspaceFiles)
+	newFolders := makeWorkspaceFolders(e.sandbox, folders, config.NoDefaultWorkspaceFiles)
 	config.WorkspaceFolders = folders
 	e.SetConfig(config)
 
